/*
 * Copyright (c) 2005, Innoopract Informationssysteme GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *  Innoopract - initial API and implementation
 */
package bug2html;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Factory for Bug instances
 * @author Elias Volanakis <evolanakis@innoopract.de>
 */
public final class BugFactory {

  private BugFactory() {
    // utility class
  }

  /** 
   * Creates a new Bug instance from an XML-Node.
   * @param bugNode 
   *        A bugzilla &lt;bug&gt;....&lt;/bug&gt; Node. 
   *        (For an example see: 
   *        https://bugs.eclipse.org/bugs/show_bug.cgi?id=1&ctype=xml)
   */
  public static Bug newBug(final Node bugNode) {
    Bug result = new Bug();
    parseBug(bugNode, result);
    return result;
  }

  private static void parseBug(final Node bugNode, final Bug bug) {
    if (bug == null || bugNode == null || !bugNode.getNodeName().equals("bug")) {
      throw new IllegalArgumentException();
    }
    // extract relevant data from childNodes
    NodeList childNodes = bugNode.getChildNodes();
    for (int i = 0, n = childNodes.getLength(); i < n; i++) {
      Node node = childNodes.item(i);
      String nodeName = node.getNodeName();
      if ("bug_id".equals(nodeName)) {
        bug
            .setId(Integer
                .parseInt(node.getChildNodes().item(0).getNodeValue()));
      } else if ("target_milestone".equals(nodeName)) {
        bug.setMilestone(node.getChildNodes().item(0).getNodeValue());
      } else if ("short_desc".equals(nodeName)) {
        bug.setShortDescr(node.getChildNodes().item(0).getNodeValue());
      } else if ("dependson".equals(nodeName)) {
        Integer bugId = new Integer(node.getChildNodes().item(0).getNodeValue());
        bug.addDependency(bugId);
      } else if ("keywords".equals(nodeName)) {
        // if-statemend needed, because multiple keywords are possible
        if ("helpWanted".equals(node.getChildNodes().item(0).getNodeValue())) {
          bug.setHelpWanted(true);
        }
      } else if ("priority".equals(nodeName)) {
        bug.setPriority(node.getChildNodes().item(0).getNodeValue());
      } else if ("bug_status".equals(nodeName)) {
        bug.setStatus(node.getChildNodes().item(0).getNodeValue());
      } else if ("component".equals(nodeName)) {
        bug.setComponent(node.getChildNodes().item(0).getNodeValue());
      } else if ("long_desc".equals(nodeName)
          && bug.getLongDescr() != Description.EMPTY_DESCRIPTION) {
        bug.setLongDescr(parseLongDescr(node, bug));
      }
    } // for
  }

  private static Description parseLongDescr(final Node ldNode, final Bug bug) {
    if (bug == null || ldNode == null || bug == null
        || !ldNode.getNodeName().equals("long_desc")) {
      throw new IllegalArgumentException();
    }
    NodeList childNodes = ldNode.getChildNodes();
    String text = "";
    String who = "";
    String when = "";
    for (int i = 0, n = childNodes.getLength(); i < n; i++) {
      Node node = childNodes.item(i);
      String nodeName = node.getNodeName();
      if ("thetext".equals(nodeName)) {
        text = node.getChildNodes().item(0).getNodeValue();
      } else if ("who".equals(nodeName)) {
        who = node.getChildNodes().item(0).getNodeValue();
      } else if ("bug_when".equals(nodeName)) {
        when = node.getChildNodes().item(0).getNodeValue();
      }
    }
    return new Description(text, who, when);
  }

}
